//+------------------------------------------------------------------+
//|                                                Gradient Test.mq5 |
//|                                      Copyright 2020, Dale Woods. |
//|                                      https://www.theforexguy.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2020, Dale Woods."
#property link      "https://www.theforexguy.com"
#property version   "1.00"
#property indicator_separate_window

#property indicator_buffers 3
#property indicator_plots 2

#property indicator_width1 6
#property indicator_style1 STYLE_SOLID
#property indicator_type1  DRAW_COLOR_HISTOGRAM
#property indicator_label1 "Black Line"
#property indicator_color1 Black


#property indicator_width2 2
#property indicator_style2 STYLE_SOLID
#property indicator_type2  DRAW_LINE
#property indicator_label2 "Black Line"
#property indicator_color2 Red

#include "Includes\ColorGradient.mqh"


int calcPeriod = 50;

double black_line[], red_line[], histogram_colors[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit() {
   ArraySetAsSeries(black_line, true);   
   ArraySetAsSeries(red_line, true);     
   ArraySetAsSeries(histogram_colors, true); 
      
   //inside candle buffers
   SetIndexBuffer(0,black_line); 
   
   SetIndexBuffer(2,red_line); 
     
   SetIndexBuffer(1,histogram_colors,INDICATOR_COLOR_INDEX);
      
   //Assign the color indexes array with indicator's buffer
   setPlotColors(0);
   
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
   const int rates_total,
   const int prev_calculated,
   const datetime &time[],
   const double &open[],
   const double &high[],
   const double &low[],
   const double &close[],
   const long &tick_volume[],
   const long &volume[],
   const int &spread[]
) {
   //---- check for the presence of bars, sufficient for the calculation
   if (rates_total < calcPeriod - 1) {
      return(0);
   }  
   
   int limit = rates_total;
   
   //---- calculation of starting index first of the main loop
   if(prev_calculated==0) { // check for the first start of the indicator 
      limit = rates_total-1; // start index for all the bars
   }
   
   else { 
      limit = rates_total-prev_calculated-1; // start index for the new bars
   }
   
  
   //---- main loop of the calculation
   for(int i = limit; i >= 0; i--) {    
         
      black_line[i] = sinWave(i);
      red_line[i] = sinWave(i, -0.5);
      
   }
   
   return rates_total;
}


double sinWave(int i, double shift = 0.0) {
    double multiplier= 2 * (M_PI/calcPeriod);    
    return sin( (i * multiplier) + shift);
}



void setPlotColors(int plot) {
   
   PlotIndexSetInteger(plot,PLOT_COLOR_INDEXES,calcPeriod); //Set number of colors
   
   ColorGradient gradient(calcPeriod);
   gradient.SetColorsMql(clrWhite, clrBlack);
   gradient.SetColorsMql(clrBlue, clrRed);
  
   //gradient.SetColorsMql(clrRed, clrGray, clrBlue);
   
   for(int i=0;i<=calcPeriod;i++) {
      PlotIndexSetInteger(plot,PLOT_LINE_COLOR,i, gradient.GetColorAt(i));
   }
   
}
//+------------------------------------------------------------------+
//|  Get index of the color                                          |
//+------------------------------------------------------------------+
/*
*       The function return the color index
*       The first parameter is the current value of the indicator
*       The second parameter is the minimal value of the indicator
*       The third parameter is the maximal value of the indicator
*/
double getPlotColor(double current,double min,double max) {
   int val = 0 ;
   
   if( min < 0 ) { 
      double shift = fabs(min);
      val = (int) (50 * NormalizeValue(current+shift,0,max+shift));
   }
   
   else {
      val = (int) (50 * NormalizeValue(current,min,max));
   }
   
   return (double) MathMin( MathMax(0, val), 49);
}



double NormalizeValue(
      const double value,           // Value to normalize
      const double min,             // Value min range value
      const double max,             // Value max range value
      const double ymin = 0,        // Normalized min value
      const double ymax = 1         // Normalized max value
   ) {
      //if( value == 0 && min == 0 ) { return ymin; }   
      if( value == 0 ) { return 0; }   
      double normed = ( ((value - min) * (ymax - ymin)) / (max - min) ) + ymin;  
      
   //   normed = MathMin( normed, ymax );
    //  normed = MathMax( normed, ymin);
      return normed;
       
      double average      = (min + max) / 2;
      double range        = (max - min);
      double normalized_x = (value - average) / range;
      return 2 * normalized_x - 1;
   } 